/*********************************************//*!@addtogroup file Files*//*@{*/
/*!
 *******************************************************************************
* File identification:      $Id: dialog_Calendar.qml 71 2016-12-31 16:42:30Z cypax $
* Revision of last commit:  $Rev: 71 $
* Author of last commit:    $Author: cypax $
* Date of last commit:      $Date: 2016-12-31 17:42:30 +0100 (Sa, 31 Dez 2016) $
 *******************************************************************************
 *
 * @author        Cypax
 *
 * @brief         Dialog window for calendar options.
 */
/************************************************************************//*@}*/

import QtQuick 2.4
import QtQuick.Window 2.2
import QtQuick.Controls 1.3
import QtQuick.Layouts 1.1
import QtQuick.Dialogs 1.2
import QtQuick.Controls.Styles 1.3
import QtGraphicalEffects 1.0

import CalendarExampleTypes 1.0


Dialog {
    id: thisDialog
    modality: Qt.WindowModal

    title: "Calendar options"

    property int calendarIndex: -1
    property color calendarColor: "#000000"


    contentItem: Rectangle {
        id: rectContentItem

        implicitWidth: 400
        implicitHeight: 250

        focus: true
        color: calendarColor

        Column {
            id: col_Fields
            spacing: 8
            anchors.fill: parent
            anchors.margins: 8

            Row { width: parent.width; spacing: 8;
                Label { width: 50; color: "#fff"; text: (calendarIndex >= 0 ? qsTr("Calendar #%1").arg(calendarIndex+1) : qsTr("New calendar")); font.bold: true; }
            }
            Row { width: parent.width; spacing: 8;
                Label { width: 50; color: "#fff"; text: qsTr("Type:"); }
                ComboBox { id: cbType; visible: (calendarIndex < 0); model: ["ICS", "calDAV"]; width: parent.width-parent.spacing-50; currentIndex: calendarIndex >= 0 ? (calendarManager.getListItemAt(calendarIndex).calendarType === CalendarClient_CalDAV.E_CALENDAR_ICS ? 0 : 1) : 0; }
                Label { visible: (calendarIndex >= 0); width: parent.width-parent.spacing-50;  color: "#fff"; text: calendarIndex >= 0 ? (calendarManager.getListItemAt(calendarIndex).calendarType === CalendarClient_CalDAV.E_CALENDAR_ICS ? "ICS" : "calDAV") : ""; }
            }
            Row { width: parent.width; spacing: 8;
                Label { width: 50; color: "#fff"; text: qsTr("Name:"); }
                TextField { id: tfName; width: parent.width-parent.spacing-50; text: calendarIndex >= 0 ? calendarManager.getListItemAt(calendarIndex).displayName : ""; }
            }
            Row { width: parent.width; spacing: 8;
                Label { width: 50; color: "#fff"; text: qsTr("URL:"); }
                TextField { id: tfURL; placeholderText: cbType.currentIndex == 0 ? "https://server.tld/folder/filename.ics" : "https://server.tld/owncloud/remote.php/dav/calendars/username/calendarname/"; width: parent.width-parent.spacing-50; text: calendarIndex >= 0 ? calendarManager.getListItemAt(calendarIndex).hostURL : ""; }
            }
            Row { width: parent.width; spacing: 8;
                Label { id: lbColor; width: 50; color: "#fff"; text: qsTr("Color:"); }
                Rectangle {
                    id: rectColor; width: parent.width-parent.spacing-50; height: lbColor.height;
                    color: calendarColor
                    border.color: "#ffffff"
                    border.width: 1
                    MouseArea {
                        anchors.fill: parent;
                        onClicked: {
                            cdColor.color = calendarColor
                            cdColor.open()
                        }
                    }
                    ColorDialog {
                        id: cdColor;
                        onAccepted: calendarColor = color
                    }
                }
            }
            Row { visible: (cbType.currentIndex === 1); width: parent.width; spacing: 8;
                Label { width: 50; color: "#fff"; text: qsTr("Username:"); }
                TextField { id: tfUsername; width: parent.width-parent.spacing-50; text: (calendarIndex >= 0 && cbType.currentIndex === 1) ? calendarManager.getListItemAt(calendarIndex).username : ""; }
            }
            Row { visible: (cbType.currentIndex === 1); width: parent.width; spacing: 8;
                Label { width: 50; color: "#fff"; text: qsTr("Password:"); }
                TextField { id: tfPassword; width: parent.width-parent.spacing-50; text: (calendarIndex >= 0 && cbType.currentIndex === 1) ? calendarManager.getListItemAt(calendarIndex).password : ""; echoMode: TextInput.Password; }
            }
            Row { width: parent.width; spacing: 8;
                Button {
                    width: (parent.width-parent.spacing)/2
                    text: qsTr("OK")
                    onClicked: {
                        if (calendarIndex < 0)
                        {
                            if (cbType.currentIndex === 0) {
                                calendarManager.addICS_Calendar(calendarColor, tfName.text, tfURL.text)
                            } else {
                                calendarManager.addCalDAV_Calendar(calendarColor, tfName.text, tfURL.text, tfUsername.text, tfPassword.text)
                            }
                        } else {
                            if (cbType.currentIndex === 0) {
                                calendarManager.getListItemAt(calendarIndex).hostURL = tfURL.text
                                calendarManager.getListItemAt(calendarIndex).displayName = tfName.text
                                calendarManager.getListItemAt(calendarIndex).color = calendarColor
                            } else {
                                calendarManager.getListItemAt(calendarIndex).hostURL = tfURL.text
                                calendarManager.getListItemAt(calendarIndex).displayName = tfName.text
                                calendarManager.getListItemAt(calendarIndex).color = calendarColor
                                calendarManager.getListItemAt(calendarIndex).username = tfUsername.text
                                calendarManager.getListItemAt(calendarIndex).password = tfPassword.text
                            }
                            calendarManager.getListItemAt(calendarIndex).startSynchronization()
                        }

                        thisDialog.close()
                    }
                }
                Button {
                    width: (parent.width-parent.spacing)/2
                    text: qsTr("Abort")
                    onClicked: { thisDialog.close() }
                }
            }

        }


        Keys.onEscapePressed: {
            thisDialog.close()
        }
        Keys.onEnterPressed: {
            thisDialog.close()
        }
        Keys.onReturnPressed: {
            thisDialog.close()
        }

    } //END: contentItem: Column



}
