/*********************************************//*!@addtogroup file Files*//*@{*/
/*!
 *******************************************************************************
* File identification:      $Id: main.qml 71 2016-12-31 16:42:30Z cypax $
* Revision of last commit:  $Rev: 71 $
* Author of last commit:    $Author: cypax $
* Date of last commit:      $Date: 2016-12-31 17:42:30 +0100 (Sa, 31 Dez 2016) $
 *******************************************************************************
 *
 * @author        Cypax
 *
 * @brief         Main GUI file of iCalendar example.
 */
/************************************************************************//*@}*/

import QtQuick 2.2
import QtQuick.Controls 1.2
import QtQuick.Controls.Private 1.0
import QtQuick.Controls.Styles 1.1
import QtGraphicalEffects 1.0

import CalendarExampleTypes 1.0

ApplicationWindow {
    visible: true
    width: 1200
    height: 600
    minimumWidth: 400
    minimumHeight: 300
    color: "#f4f4f4"

    title: "iCalendar example 1.1.0.0"

    Component {
        id: calstyle
        CalendarStyle {
            dayDelegate: Item {
                id: thisItem
                readonly property color sameMonthDateTextColor: "#444"
                readonly property color selectedDateColor: "#403778d0"
                readonly property color selectedDateTextColor: "white"
                readonly property color differentMonthDateTextColor: "#bbb"
                readonly property color invalidDatecolor: "#dddddd"

                Rectangle {
                    anchors.fill: parent
                    border.color: "transparent"
                    color: styleData.date !== undefined && styleData.selected ? selectedDateColor : "transparent"
                    anchors.margins: styleData.selected ? -1 : 0
                }


                Text {
                    id: dayDelegateText
                    text: styleData.date.getDate()
                    anchors {
                        top: parent.top
                        bottom: parent.bottom
                        left: parent.left
                        margins: 4
                    }
                    width: parent.width / 3
                    font.bold: true
                    color: {
                        var color = invalidDatecolor;
                        if (styleData.valid) {
                            // Date is within the valid range.
                            color = styleData.visibleMonth ? sameMonthDateTextColor : differentMonthDateTextColor;
                            if (styleData.selected) {
                                color = selectedDateTextColor;
                            }
                        }
                        color;
                    }
                }

                Grid  {
                    id: thisGrid
                    anchors {
                        top: parent.top
                        bottom: parent.bottom
                        left: dayDelegateText.right
                        right: parent.right
                    }

                    anchors.margins: 4
                    columns: 2
                    rows: 4
                    spacing: 4
                    readonly property real itemWidth: (thisGrid.width - (thisGrid.spacing*thisGrid.columns))/thisGrid.columns
                    readonly property real itemHeight: (thisGrid.height - (thisGrid.spacing*thisGrid.rows))/thisGrid.rows

                    function redrawCalendarFlags() {
                        var evtList = calendarManager.eventsForDate(styleData.date)
                        thisListModel.clear()
                        for (var ci = 0; ci < calendarManager.listOfCalendars.length; ci++) {
                            for (var ei = 0; ei < evtList.length; ei++) {
                                if ( (false === evtList[ei].isCanceled) && (calendarManager.getListItemAt(ci).displayName == evtList[ei].calendarName ) ) {
                                    thisListModel.append({"calendarName":calendarManager.getListItemAt(ci).displayName, "calendarColor":calendarManager.getListItemAt(ci).color})
                                    break;
                                }
                            }
                        }
                    }

                    Connections {
                        target: calendarManager
                        onEventsUpdated: {
                            thisGrid.redrawCalendarFlags()
                        }
                        onListOfCalendarsChanged: {
                            thisGrid.redrawCalendarFlags()
                        }
                    }

                    ListModel {
                        id: thisListModel
                    }

                    Repeater {
                        id: rep_DayCalendars
                        delegate: Item {
                            width: thisGrid.itemWidth
                            height: thisGrid.itemHeight
                            Image {
                                id: thisImage
                                anchors.fill: parent
                                visible: false
                                source: "qrc:///resources/images/icon_flag.svg"
                            }
                            ColorOverlay {
                                anchors.fill: parent
                                source: thisImage
                                color: calendarColor
                            }
                        }
                        model: thisListModel
                    }
                }
            }
        }
    }

    Flow {
        id: flow_Content
        anchors.fill: parent
        anchors.margins: 20
        spacing: 10
        layoutDirection: Qt.RightToLeft

        Column {
            id: col_Calendar
            width: (parent.width > parent.height ? parent.width * 0.5 - parent.spacing : parent.width)
            height: (parent.height > parent.width ? parent.height * 0.5 - parent.spacing : parent.height)

            Calendar {
                id: calendar
                width: parent.width
                height: parent.height
                frameVisible: true
                weekNumbersVisible: true
                selectedDate: new Date()
                focus: true
                style: calstyle
                onVisibleMonthChanged: {
                    calendarManager.date = new Date(calendar.visibleYear, calendar.visibleMonth, 1)
                }
                onVisibleYearChanged: {
                    calendarManager.date = new Date(calendar.visibleYear, calendar.visibleMonth, 1)
                }
            }

        } // END: col_Calendar

        Component {
            id: comp_EventListHeader

            Row {
                id: eventDateRow
                width: parent.width
                height: eventDayLabel.height
                spacing: 10

                Label {
                    id: eventDayLabel
                    text: calendar.selectedDate.getDate()
                    font.pointSize: 35
                }

                Column {
                    height: eventDayLabel.height

                    Label {
                        readonly property var options: { weekday: "long" }
                        text: Qt.locale().standaloneDayName(calendar.selectedDate.getDay(), Locale.LongFormat)
                        font.pointSize: 18
                    }
                    Label {
                        text: Qt.locale().standaloneMonthName(calendar.selectedDate.getMonth())
                              + calendar.selectedDate.toLocaleDateString(Qt.locale(), " yyyy")
                        font.pointSize: 12
                    }
                }
            }
        } // END: comp_EventListHeader

        Rectangle {
            id: rect_EventList
            width: (parent.width > parent.height ? parent.width * 0.25 - parent.spacing : parent.width)
            height: (parent.height > parent.width ? parent.height * 0.25 - parent.spacing : parent.height)
            border.color: Qt.darker(color, 1.2)

            ListView {
                id: eventsListView
                spacing: 4
                clip: true
                header: comp_EventListHeader
                anchors.fill: parent
                anchors.margins: 10

                Connections {
                    target: calendarManager
                    onEventsUpdated: {
                        eventsListView.model = calendarManager.eventsForDate(calendar.selectedDate)
                    }
                    onListOfCalendarsChanged: {
                        eventsListView.model = calendarManager.eventsForDate(calendar.selectedDate)
                    }
                }
                Connections {
                    target: calendar
                    onSelectedDateChanged: {
                        eventsListView.model = calendarManager.eventsForDate(calendar.selectedDate)
                    }
                }

                delegate: Rectangle {
                    width: eventsListView.width
                    height: eventItemColumn.height
                    anchors.horizontalCenter: parent.horizontalCenter

                    MouseArea {
                        anchors.fill: parent
                        onClicked: {
                            descriptionLabel.visible = !descriptionLabel.visible
                        }
                    }

                    Image {
                        id: evtImage
                        anchors.top: parent.top
                        anchors.topMargin: 4
                        width: 12
                        height: width
                        source: "qrc:///resources/images/icon_flag.svg"
                        visible: false
                    }
                    ColorOverlay {
                        anchors.fill: evtImage
                        source: evtImage
                        color: modelData.color
                        rotation: descriptionLabel.visible ? 180 : 0
                    }

                    Rectangle {
                        width: parent.width
                        height: 1
                        color: "#eee"
                    }

                    Image {
                        id: evtEdit
                        anchors.top: parent.top
                        anchors.topMargin: 4
                        anchors.right: parent.right
                        width: 20
                        height: 27
                        source: "qrc:///resources/images/icon_edit.svg"
                        visible: modelData.getCalendarPointer().calendarType === CalendarClient_CalDAV.E_CALENDAR_CALDAV
                        opacity: 0.3
                        MouseArea {
                            anchors.fill: parent
                            hoverEnabled: true
                            onEntered: { evtEdit.opacity = 1.0; }
                            onExited: {evtEdit.opacity = 0.3; }
                            onClicked: {
                                eventDialog.clearOptions()
                                eventDialog.evtSummary = modelData.name
                                eventDialog.evtLocation = modelData.location
                                eventDialog.evtDescription = modelData.description
                                eventDialog.calendarColor = modelData.color
                                eventDialog.evtExdate = modelData.exdates
                                eventDialog.evtRrule = modelData.rrule
                                eventDialog.evtUID = modelData.uid
                                eventDialog.evtHREF = modelData.href
                                eventDialog.evtStartDateTime = modelData.startDateTime
                                eventDialog.evtEndDateTime = modelData.endDateTime
                                eventDialog.calendarPtr = modelData.getCalendarPointer()

                                eventDialog.open()
                            }
                        }
                    }

                    Column {
                        id: eventItemColumn
                        anchors.left: parent.left
                        anchors.leftMargin: 20
                        anchors.right: evtEdit.left
                        height: timeLabel.height + nameLabel.height + (descriptionLabel.visible ? descriptionLabel.height : 0) + (locationLabel.visible ? locationLabel.height : 0) + 8

                        Row {
                            Label {
                                wrapMode: Text.Wrap
                                text: qsTr("CANCELED: ")
                                visible: modelData.isCanceled
                                color: "#990000"
                            }
                            Label {
                                id: nameLabel
                                wrapMode: Text.Wrap
                                text: qsTr("%1: %2").arg(modelData.calendarName).arg(modelData.name)
                                font.strikeout: modelData.isCanceled
                            }
                        }

                        Label {
                            id: timeLabel
                            width: parent.width
                            wrapMode: Text.Wrap
                            text: (modelData.startDateTime.getHours() === 0 && modelData.startDateTime.getMinutes() === 0 && ( (modelData.endDateTime.getHours() === 0 && modelData.endDateTime.getMinutes() === 0) || (modelData.endDateTime.getHours() === 23 && modelData.endDateTime.getMinutes() === 59) ) ) ? qsTr("all-day") : ((modelData.startDateTime.getHours() === modelData.endDateTime.getHours()) && (modelData.startDateTime.getMinutes() === modelData.endDateTime.getMinutes())) ? qsTr("Starting %1").arg(modelData.startDateTime.toLocaleTimeString(calendar.locale, Locale.ShortFormat)) : qsTr("%1 - %2").arg(modelData.startDateTime.toLocaleTimeString(calendar.locale, Locale.ShortFormat)).arg(modelData.endDateTime.toLocaleTimeString(calendar.locale, Locale.ShortFormat))
                            color: "#666"
                            font.strikeout: modelData.isCanceled
                        }
                        Label {
                            id: locationLabel
                            width: parent.width
                            wrapMode: Text.Wrap
                            text: qsTr("Location: %1").arg(modelData.location)
                            color: "#666"
                            visible: descriptionLabel.visible
                            font.strikeout: modelData.isCanceled
                        }
                        Label {
                            id: descriptionLabel
                            width: parent.width
                            wrapMode: Text.Wrap
                            text: qsTr("%1").arg(modelData.description)
                            color: "#666"
                            visible: false
                            font.strikeout: modelData.isCanceled
                        }

                    }

                }
            }
        } // END: rect_EventList

        Component {
            id: comp_CalendarSource

            Rectangle {
                width: parent.width
                height: 150
                color: calendarManager.getListItemAt(index).color
                Column {
                    id: col_CalSource
                    anchors {
                        left: parent.left
                    }
                    width: parent.width * 0.75
                    anchors.margins: 4
                    spacing: 4
                    Row { width: parent.width; spacing: 8; Label { width: 50; color: "#fff"; text: "Calendar #" +  (index+1); font.bold: true; }  }
                    Row { width: parent.width; spacing: 8; Label { width: 50; color: "#fff"; text: "Type:"; } Label { color: "#fff"; clip: true; width: parent.width-parent.spacing-50; text: calendarManager.getListItemAt(index).calendarType === CalendarClient_CalDAV.E_CALENDAR_ICS ? "ICS" : "CalDAV"; } }
                    Row { width: parent.width; spacing: 8; Label { width: 50; color: "#fff"; text: "Name:"; } Label { color: "#fff"; clip: true; width: parent.width-parent.spacing-50; text: calendarManager.getListItemAt(index).displayName; } }
                    Row { width: parent.width; spacing: 8; Label { width: 50; color: "#fff"; text: "URL:"; } Label { color: "#fff"; clip: true; width: parent.width-parent.spacing-50; text: calendarManager.getListItemAt(index).hostURL; } }
                    Button { text: qsTr("remove"); width: 80; onClicked: { calendarManager.removeListItemAt(index) } }
                    Button { text: qsTr("edit"); width: 80; onClicked: { calendarDialog.calendarColor = calendarManager.getListItemAt(index).color; calendarDialog.calendarIndex = index; calendarDialog.open() } }
                    Button {
                        id: btn_AddEvent
                        text: qsTr("Add Event")
                        width: 80
                        visible: calendarManager.getListItemAt(index).calendarType === CalendarClient_CalDAV.E_CALENDAR_CALDAV
                        onClicked: {
                            eventDialog.clearOptions()
                            eventDialog.calendarColor = calendarManager.getListItemAt(index).color
                            eventDialog.calendarPtr = calendarManager.getListItemAt(index)
                            eventDialog.open()
                        }
                    }

                }

                Item {
                    anchors {
                        left: col_CalSource.right
                        right: parent.right
                        top: parent.top
                        bottom: parent.bottom
                    }

                    BusyIndicator {
                        id: bi_SyncState
                        running: (calendarManager.getListItemAt(index).syncState === CalendarClient.E_STATE_BUSY)
                        anchors.right: parent.right
                        anchors.rightMargin: 8
                        anchors.bottom: parent.bottom
                        anchors.bottomMargin: 8
                    }

                    Image {
                        visible: (calendarManager.getListItemAt(index).syncState === CalendarClient.E_STATE_ERROR)
                        anchors.right: parent.right
                        anchors.rightMargin: 8
                        anchors.bottom: parent.bottom
                        anchors.bottomMargin: 8
                        width: bi_SyncState.width
                        height: bi_SyncState.height
                        source: "qrc:///resources/images/icon_error.svg"
                        MouseArea {
                            anchors.fill: parent
                            onPressed: {
                                // try to resolve error
                                calendarManager.getListItemAt(index).recover()
                            }
                        }
                    }
                }
            }


        } // END: comp_CalendarSource

         CalendarDialog {
            id: calendarDialog
         }

        Column {
            id: col_CalendarSources
            width: (parent.width > parent.height ? parent.width * 0.25 - parent.spacing : parent.width)
            height: (parent.height > parent.width ? parent.height * 0.25 - parent.spacing : parent.height)
            spacing: 16

            ListView {
                id: lv_CalendarSources
                height: parent.height - parent.spacing - btn_AddCalendar.height
                width: parent.width
                delegate: comp_CalendarSource
                model: calendarManager.listOfCalendars
                clip: true
                focus: true
                spacing: 16
            } // END: lv_CalendarSources

            EventDialog {
                id: eventDialog
            }

            Button {
                id: btn_AddCalendar
                text: qsTr("Add calendar")
                width: parent.width
                onClicked: {
                    calendarDialog.calendarColor = "#005500"
                    calendarDialog.calendarIndex = -1
                    calendarDialog.open()
                }
            }

        } // END: col_CalendarSources

    } // END: flow_Content
}
